@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Id: autogen.sh 5312 2022-02-01 18:39:20Z chrfranke $
+# $Id: autogen.sh 5632 2024-10-26 10:49:43Z chrfranke $
 #
 # Generate ./configure from configure.ac and Makefile.in from Makefile.am.
 # This also adds files like missing,depcomp,install-sh to the source
@@ -13,22 +13,17 @@ while [ $# -gt 0 ]; do case $1 in
   *) echo "Usage: $0 [--force] [--warnings=CATEGORY ...]"; exit 1 ;;
 esac; done
 
-# Cygwin?
-test -x /usr/bin/uname && /usr/bin/uname | grep -i CYGWIN >/dev/null &&
-{
-    # Check for Unix text file type
-    echo > dostest.tmp
-    test "`wc -c < dostest.tmp`" -eq 1 ||
-        echo "Warning: DOS text file type set, 'make dist' and related targets will not work."
-    rm -f dostest.tmp
-}
+# Check for CR/LF line endings
+if od -A n -t x1 smartctl.h | grep ' 0d' >/dev/null; then
+  echo "Warning: Checkout with CR/LF line endings, 'make dist' and related targets will not work."
+fi
 
 # Find automake
 if [ -n "$AUTOMAKE" ]; then
   ver=$("$AUTOMAKE" --version) || exit 1
 else
   maxver=
-  for v in 1.16 1.15 1.14 1.13 1.12 1.11 1.10; do
+  for v in 1.17 1.16 1.15 1.14 1.13; do
     minver=$v; test -n "$maxver" || maxver=$v
     ver=$(automake-$v --version 2>/dev/null) || continue
     AUTOMAKE="automake-$v"
@@ -36,7 +31,7 @@ else
   done
   if [ -z "$AUTOMAKE" ]; then
     echo "GNU Automake $minver (up to $maxver) is required to bootstrap smartmontools from SVN."
-    exit 1;
+    exit 1
   fi
 fi
 
@@ -56,11 +51,15 @@ fi
 # Warn if Automake version was not tested
 amwarnings=$warnings
 case "$ver" in
-  1.10|1.10.[123]|1.11|1.11.[1-6]|1.12.[2-6]|1.13.[34])
+  1.[0-9]|1.[0-9].*|1.1[0-2]|1.1[0-2].*)
+    echo "GNU Automake $ver is not supported."; exit 1
+    ;;
+
+  1.13.[34])
     # OK
     ;;
 
-  1.14|1.14.1|1.15|1.15.1|1.16|1.16.[1-5])
+  1.14|1.14.1|1.15|1.15.1|1.16|1.16.[1-5]|1.17)
     # TODO: Enable 'subdir-objects' in configure.ac
     # For now, suppress 'subdir-objects' forward-incompatibility warning
     test -n "$warnings" || amwarnings="--warnings=no-unsupported"
@@ -71,9 +70,6 @@ case "$ver" in
     echo "Please report success/failure to the smartmontools-support mailing list."
 esac
 
-# required for aclocal-1.10 --install
-test -d m4 || mkdir m4 || exit 1
-
 set -e	# stops on error status
 
 test -z "$warnings" || set -x
