@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2002-8 Bruce Allen
  * Copyright (C) 2000 Michael Cornwell <cornwell@acm.org>
- * Copyright (C) 2003-18 Douglas Gilbert <dgilbert@interlog.com>
+ * Copyright (C) 2003-2023 Douglas Gilbert <dgilbert@interlog.com>
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
@@ -19,7 +19,7 @@
 #ifndef SCSICMDS_H_
 #define SCSICMDS_H_
 
-#define SCSICMDS_H_CVSID "$Id: scsicmds.h 5337 2022-02-27 07:53:55Z dpgilbert $\n"
+#define SCSICMDS_H_CVSID "$Id: scsicmds.h 5462 2023-03-13 10:45:06Z chrfranke $\n"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,14 +39,14 @@
 #ifndef LOG_SENSE
 #define LOG_SENSE 0x4d
 #endif
-#ifndef MODE_SENSE
-#define MODE_SENSE 0x1a
+#ifndef MODE_SENSE_6
+#define MODE_SENSE_6 0x1a
 #endif
 #ifndef MODE_SENSE_10
 #define MODE_SENSE_10 0x5a
 #endif
-#ifndef MODE_SELECT
-#define MODE_SELECT 0x15
+#ifndef MODE_SELECT_6
+#define MODE_SELECT_6 0x15
 #endif
 #ifndef MODE_SELECT_10
 #define MODE_SELECT_10 0x55
@@ -78,12 +78,24 @@
 #ifndef READ_CAPACITY_10
 #define READ_CAPACITY_10  0x25
 #endif
-#ifndef READ_CAPACITY_16
-#define READ_CAPACITY_16  0x9e
+#ifndef SERVICE_ACTION_IN_16
+#define SERVICE_ACTION_IN_16  0x9e
 #endif
-#ifndef SAI_READ_CAPACITY_16    /* service action for READ_CAPACITY_16 */
+#ifndef SAI_READ_CAPACITY_16    /* service action in for READ_CAPACITY_16 */
 #define SAI_READ_CAPACITY_16  0x10
 #endif
+#ifndef SAI_GET_PHY_ELEM_STATUS    /* Get physical element status */
+#define SAI_GET_PHY_ELEM_STATUS  0x17
+#endif
+#ifndef SAI_REPORT_SUPPORTED_OPCODES
+#define SAI_REPORT_SUPPORTED_OPCODES  0xc
+#endif
+#ifndef MAINTENANCE_IN_12
+#define MAINTENANCE_IN_12  0xa3
+#endif
+#ifndef MI_REP_SUP_OPCODES
+#define MI_REP_SUP_OPCODES  0xc    /* maintenance in (12) */
+#endif
 
 #ifndef SAT_ATA_PASSTHROUGH_12
 #define SAT_ATA_PASSTHROUGH_12 0xa1
@@ -97,6 +109,9 @@
 #define DXFER_FROM_DEVICE 1
 #define DXFER_TO_DEVICE   2
 
+
+/* scsi_rsoc_elem and scsi_device is defined in dev_interface.h */
+
 struct scsi_cmnd_io
 {
     uint8_t * cmnd;     /* [in]: ptr to SCSI command block (cdb) */
@@ -175,9 +190,12 @@ struct scsi_supp_log_pages {
 /* SCSI Peripheral types (of interest) */
 #define SCSI_PT_DIRECT_ACCESS           0x0
 #define SCSI_PT_SEQUENTIAL_ACCESS       0x1
+#define SCSI_PT_WO                      0x4      /* write once device */
 #define SCSI_PT_CDROM                   0x5
+#define SCSI_PT_OPTICAL                 0x7
 #define SCSI_PT_MEDIUM_CHANGER          0x8
 #define SCSI_PT_ENCLOSURE               0xd
+#define SCSI_PT_RBC                     0xe
 #define SCSI_PT_HOST_MANAGED            0x14    /* Zoned disk */
 
 /* Transport protocol identifiers or just Protocol identifiers */
@@ -237,8 +255,12 @@ struct scsi_supp_log_pages {
 
 /* Seagate vendor specific log pages. */
 #define SEAGATE_CACHE_LPAGE                 0x37
+#define SEAGATE_FARM_LPAGE                  0x3d
 #define SEAGATE_FACTORY_LPAGE               0x3e
 
+/* Seagate vendor specific log sub-pages. */
+#define SEAGATE_FARM_CURRENT_L_SPAGE        0x3     /* 0x3d,0x3 */
+
 /* Log page response lengths */
 #define LOG_RESP_SELF_TEST_LEN 0x194
 
@@ -360,8 +382,6 @@ Documentation, see http://www.storage.ibm.com/techsup/hddtech/prodspecs.htm */
 #define SCSI_TIMEOUT_SELF_TEST  (5 * 60 * 60)   /* allow max 5 hours for */
                                             /* extended foreground self test */
 
-
-
 #define LOGPAGEHDRSIZE  4
 
 class scsi_device;
@@ -470,6 +490,10 @@ int scsiReadCapacity10(scsi_device * device, unsigned int * last_lbp,
 
 int scsiReadCapacity16(scsi_device * device, uint8_t *pBuf, int bufLen);
 
+int scsiRSOCcmd(scsi_device * device, bool rctd, uint8_t rep_opt,
+		uint8_t opcode, uint16_t serv_act, uint8_t *pBuf, int bufLen,
+	        int & rspLen);
+
 /* SMART specific commands */
 int scsiCheckIE(scsi_device * device, int hasIELogPage, int hasTempLogPage,
                 uint8_t *asc, uint8_t *ascq, uint8_t *currenttemp,
@@ -482,9 +506,11 @@ int scsi_IsWarningEnabled(const struct scsi_iec_mode_page *iecp);
 int scsiSetExceptionControlAndWarning(scsi_device * device, int enabled,
                             const struct scsi_iec_mode_page *iecp);
 void scsiDecodeErrCounterPage(unsigned char * resp,
-                              struct scsiErrorCounter *ecp);
+                              struct scsiErrorCounter *ecp,
+                              int allocLen);
 void scsiDecodeNonMediumErrPage(unsigned char * resp,
-                                struct scsiNonMediumError *nmep);
+                                struct scsiNonMediumError *nmep,
+                                int allocLen);
 int scsiFetchExtendedSelfTestTime(scsi_device * device, int * durationSec,
                                   int modese_len);
 int scsiCountFailedSelfTests(scsi_device * device, int noisy);
@@ -500,7 +526,7 @@ uint64_t scsiGetSize(scsi_device * device, bool avoid_rcap16,
                      struct scsi_readcap_resp * srrp);
 
 /* T10 Standard IE Additional Sense Code strings taken from t10.org */
-const char* scsiGetIEString(uint8_t asc, uint8_t ascq);
+char * scsiGetIEString(uint8_t asc, uint8_t ascq, char * b, int blen);
 int scsiGetTemp(scsi_device * device, uint8_t *currenttemp, uint8_t *triptemp);
 
 
@@ -514,14 +540,29 @@ int scsiSmartSelfTestAbort(scsi_device * device);
 const char * scsiTapeAlertsTapeDevice(unsigned short code);
 const char * scsiTapeAlertsChangerDevice(unsigned short code);
 
-const char * scsi_get_opcode_name(uint8_t opcode);
+const char * scsi_get_opcode_name(const uint8_t * cdbp);
 void scsi_format_id_string(char * out, const uint8_t * in, int n);
 
+/* Read binary starting at 'up' for 'len' bytes and output as ASCII
+ * hexadecimal into pout(). 16 bytes per line are output with an
+ * additional space between 8th and 9th byte on each line (for readability).
+ * 'no_ascii' selects one of 3 output format types:
+ *     > 0     each line has address then up to 16 ASCII-hex bytes
+ *     = 0     in addition, the bytes are rendered in ASCII to the right
+ *             of each line, non-printable characters shown as '.'
+ *     < 0     only the ASCII-hex bytes are listed (i.e. without address) */
 void dStrHex(const uint8_t * up, int len, int no_ascii);
 
+/* Read binary starting at 'up' for 'len' bytes and output as ASCII
+ * hexadecimal into FILE pointer (fp). If fp is nullptr, then send to
+ * pout(). Note that 'stdout' and 'stderr' can be given for 'fp'.
+ * See dStrHex() above for more information. */
+void dStrHexFp(const uint8_t * up, int len, int no_ascii, FILE * fp);
+
 /* Attempt to find the first SCSI sense data descriptor that matches the
-   given 'desc_type'. If found return pointer to start of sense data
-   descriptor; otherwise (including fixed format sense data) returns NULL. */
+ * given 'desc_type'. If found return pointer to start of sense data
+ * descriptor; otherwise (including fixed format sense data) returns
+ * nullptr. */
 const unsigned char * sg_scsi_sense_desc_find(const unsigned char * sensep,
                                               int sense_len, int desc_type);
 
