@@ -3,7 +3,7 @@
  *
  * Home page of code is: https://www.smartmontools.org
  *
- * Copyright (C) 2020-21 Christian Franke
+ * Copyright (C) 2020-25 Christian Franke
  * Copyright (C) 2018 Harry Mallon <hjmallon@gmail.com>
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -20,23 +20,67 @@
 
 #include <errno.h>
 
-const char * scsinvme_cpp_svnid = "$Id: scsinvme.cpp 5337 2022-02-27 07:53:55Z dpgilbert $";
+const char * scsinvme_cpp_svnid = "$Id: scsinvme.cpp 5677 2025-03-20 17:22:18Z chrfranke $";
 
 // SNT (SCSI NVMe Translation) namespace and prefix
 namespace snt {
 
 /////////////////////////////////////////////////////////////////////////////
-// sntasmedia_device
+// nvme_or_sat_device: Common base class for NVMe/SATA -> USB bridges
 
-class sntasmedia_device
+class nvme_or_sat_device
 : public tunnelled_device<
     /*implements*/ nvme_device,
     /*by tunnelling through a*/ scsi_device
   >
 {
+public:
+  nvme_or_sat_device(scsi_device * scsidev, unsigned nsid, bool maybe_sat);
+
+  virtual smart_device * autodetect_open() override;
+
+private:
+  bool m_maybe_sat;
+};
+
+nvme_or_sat_device::nvme_or_sat_device(scsi_device * scsidev, unsigned nsid, bool maybe_sat)
+: smart_device(never_called),
+  tunnelled_device<nvme_device, scsi_device>(scsidev, nsid),
+  m_maybe_sat(maybe_sat)
+{
+}
+
+smart_device * nvme_or_sat_device::autodetect_open()
+{
+  if (!open() || !m_maybe_sat)
+    return this;
+
+  // SAT not tried first because some USB bridges emulate ATA IDENTIFY via SAT
+  // if a NVMe device is connected.
+  // TODO: Preserve id_ctrl for next nvme_read_id_ctrl() call
+  smartmontools::nvme_id_ctrl id_ctrl;
+  if (nvme_read_id_ctrl(this, id_ctrl))
+    return this;
+
+  // NVMe Identify Controller failed, use the already opened SCSI device for SAT.
+  // IMPORTANT for derived classes: this->close() not called before delete.
+  // TODO: preserve requested 'type'.
+  scsi_device * scsidev = get_tunnel_dev();
+  ata_device * satdev = smi()->get_sat_device("sat", scsidev);
+  release(scsidev); // 'scsidev' is now owned by 'satdev'
+  delete this;
+  return satdev;
+}
+
+/////////////////////////////////////////////////////////////////////////////
+// sntasmedia_device
+
+class sntasmedia_device
+: public nvme_or_sat_device
+{
 public:
   sntasmedia_device(smart_interface * intf, scsi_device * scsidev,
-                    const char * req_type, unsigned nsid);
+                    const char * req_type, unsigned nsid, bool maybe_sat);
 
   virtual ~sntasmedia_device();
 
@@ -44,9 +88,9 @@ public:
 };
 
 sntasmedia_device::sntasmedia_device(smart_interface * intf, scsi_device * scsidev,
-                                     const char * req_type, unsigned nsid)
+                                     const char * req_type, unsigned nsid, bool maybe_sat)
 : smart_device(intf, scsidev->get_dev_name(), "sntasmedia", req_type),
-  tunnelled_device<nvme_device, scsi_device>(scsidev, nsid)
+  nvme_or_sat_device(scsidev, nsid, maybe_sat)
 {
   set_info().info_name = strprintf("%s [USB NVMe ASMedia]", scsidev->get_info_name());
 }
@@ -70,30 +114,32 @@ bool sntasmedia_device::nvme_pass_through(const nvme_cmd_in & in, nvme_cmd_out &
       }
       return set_err(ENOSYS, "NVMe Identify with CDW10=0x%08x not supported", in.cdw10);
     case smartmontools::nvme_admin_get_log_page:
-      if (!(in.nsid == 0xffffffff || !in.nsid))
+      if (!(in.nsid == nvme_broadcast_nsid || !in.nsid))
         return set_err(ENOSYS, "NVMe Get Log Page with NSID=0x%x not supported", in.nsid);
-      if (size > 0x200) { // Reading more results in command timeout
-        // TODO: Add ability to return short reads to caller
-        size = 0x200;
-        cdw10_hi = (size / 4) - 1;
-        pout("Warning: NVMe Get Log truncated to 0x%03x bytes, 0x%03x bytes zero filled\n", size, in.size - size);
-      }
       break;
     default:
       return set_err(ENOSYS, "NVMe admin command 0x%02x not supported", in.opcode);
     break;
   }
-  if (in.cdw11 || in.cdw12 || in.cdw13 || in.cdw14 || in.cdw15)
-    return set_err(ENOSYS, "Nonzero NVMe command dwords 11-15 not supported");
+  if (in.cdw11 || in.cdw14 || in.cdw15)
+    return set_err(ENOSYS, "Nonzero NVMe command dwords 11, 14, or 15 not supported");
 
   uint8_t cdb[16] = {0, };
   cdb[0] = 0xe6;
   cdb[1] = in.opcode;
-  //cdb[2] = ?
+  //cdb[2] = 0
   cdb[3] = (uint8_t)in.cdw10;
-  //cdb[4..6] = ?
+  //cdb[4..5] = 0
+  cdb[6] = (uint8_t)(cdw10_hi >> 8);
   cdb[7] = (uint8_t)cdw10_hi;
-  //cdb[8..15] = ?
+  cdb[8] = (uint8_t)(in.cdw13 >> 24);
+  cdb[9] = (uint8_t)(in.cdw13 >> 16);
+  cdb[10] = (uint8_t)(in.cdw13 >> 8);
+  cdb[11] = (uint8_t)in.cdw13;
+  cdb[12] = (uint8_t)(in.cdw12 >> 24);
+  cdb[13] = (uint8_t)(in.cdw12 >> 16);
+  cdb[14] = (uint8_t)(in.cdw12 >> 8);
+  cdb[15] = (uint8_t)in.cdw12;
 
   scsi_cmnd_io io_hdr = {};
   io_hdr.cmnd = cdb;
@@ -119,19 +165,14 @@ bool sntasmedia_device::nvme_pass_through(const nvme_cmd_in & in, nvme_cmd_out &
 #define SNT_JMICRON_NVM_CMD_LEN 512
 
 class sntjmicron_device
-: public tunnelled_device<
-    /*implements*/ nvme_device,
-    /*by tunnelling through a*/ scsi_device
-  >
+: public nvme_or_sat_device
 {
 public:
   sntjmicron_device(smart_interface * intf, scsi_device * scsidev,
-                    const char * req_type, unsigned nsid);
+                    const char * req_type, unsigned nsid, bool maybe_sat);
 
   virtual ~sntjmicron_device();
 
-  virtual bool open() override;
-
   virtual bool nvme_pass_through(const nvme_cmd_in & in, nvme_cmd_out & out) override;
 
 private:
@@ -142,9 +183,9 @@ private:
 };
 
 sntjmicron_device::sntjmicron_device(smart_interface * intf, scsi_device * scsidev,
-                                     const char * req_type, unsigned nsid)
+                                     const char * req_type, unsigned nsid, bool maybe_sat)
 : smart_device(intf, scsidev->get_dev_name(), "sntjmicron", req_type),
-  tunnelled_device<nvme_device, scsi_device>(scsidev, nsid)
+  nvme_or_sat_device(scsidev, nsid, maybe_sat)
 {
   set_info().info_name = strprintf("%s [USB NVMe JMicron]", scsidev->get_info_name());
 }
@@ -153,22 +194,6 @@ sntjmicron_device::~sntjmicron_device()
 {
 }
 
-bool sntjmicron_device::open()
-{
-  // Open USB first
-  if (!tunnelled_device<nvme_device, scsi_device>::open())
-    return false;
-
-  // No sure how multiple namespaces come up on device so we
-  // cannot detect e.g. /dev/sdX is NSID 2.
-  // Set to broadcast if not available
-  if (!get_nsid()) {
-    set_nsid(0xFFFFFFFF);
-  }
-
-  return true;
-}
-
 // cdb[0]: ATA PASS THROUGH (12) SCSI command opcode byte (0xa1)
 // cdb[1]: [ is admin cmd: 1 ] [ protocol : 7 ]
 // cdb[2]: reserved
@@ -188,6 +213,17 @@ bool sntjmicron_device::nvme_pass_through(const nvme_cmd_in & in, nvme_cmd_out &
 
   // 1: "NVM Command Set Payload"
   {
+  // for whatever reason selftest log causing controller to hang if size is set > 0x230b
+  // see GH issue #256 for the details. Patching it to include last 19 log records instead
+    unsigned cdw10 = in.cdw10;
+    if (in.opcode == smartmontools::nvme_admin_get_log_page) {
+      unsigned int lid = in.cdw10 & 0xFF;
+      if (lid == 0x6 && in.size > 0x218) {
+        unsigned size = 0x218;
+        cdw10 = lid | ((size/4 - 1) << 16);
+        pout("Warning: self-test output truncated to 19 items to workaround controller bug\n");
+      }
+    }
     unsigned char cdb[SNT_JMICRON_CDB_LEN] = { 0 };
     cdb[0] = SAT_ATA_PASSTHROUGH_12;
     cdb[1] = (admin ? 0x80 : 0x00) | proto_nvm_cmd;
@@ -201,7 +237,7 @@ bool sntjmicron_device::nvme_pass_through(const nvme_cmd_in & in, nvme_cmd_out &
     // nvm_cmd[4-5]: reserved
     // nvm_cmd[6-7]: metadata pointer
     // nvm_cmd[8-11]: data ptr (?)
-    nvm_cmd[12] = in.cdw10;
+    nvm_cmd[12] = cdw10;
     nvm_cmd[13] = in.cdw11;
     nvm_cmd[14] = in.cdw12;
     nvm_cmd[15] = in.cdw13;
@@ -311,14 +347,11 @@ bool sntjmicron_device::nvme_pass_through(const nvme_cmd_in & in, nvme_cmd_out &
 // sntrealtek_device
 
 class sntrealtek_device
-: public tunnelled_device<
-    /*implements*/ nvme_device,
-    /*by tunnelling through a*/ scsi_device
-  >
+: public nvme_or_sat_device
 {
 public:
   sntrealtek_device(smart_interface * intf, scsi_device * scsidev,
-                    const char * req_type, unsigned nsid);
+                    const char * req_type, unsigned nsid, bool maybe_sat);
 
   virtual ~sntrealtek_device();
 
@@ -326,9 +359,9 @@ public:
 };
 
 sntrealtek_device::sntrealtek_device(smart_interface * intf, scsi_device * scsidev,
-                                     const char * req_type, unsigned nsid)
+                                     const char * req_type, unsigned nsid, bool maybe_sat)
 : smart_device(intf, scsidev->get_dev_name(), "sntrealtek", req_type),
-  tunnelled_device<nvme_device, scsi_device>(scsidev, nsid)
+  nvme_or_sat_device(scsidev, nsid, maybe_sat)
 {
   set_info().info_name = strprintf("%s [USB NVMe Realtek]", scsidev->get_info_name());
 }
@@ -351,7 +384,7 @@ bool sntrealtek_device::nvme_pass_through(const nvme_cmd_in & in, nvme_cmd_out &
       }
       return set_err(ENOSYS, "NVMe Identify with CDW10=0x%08x not supported", in.cdw10);
     case smartmontools::nvme_admin_get_log_page:
-      if (!(in.nsid == 0xffffffff || !in.nsid))
+      if (!(in.nsid == nvme_broadcast_nsid || !in.nsid))
         return set_err(ENOSYS, "NVMe Get Log Page with NSID=0x%x not supported", in.nsid);
       if (size > 0x200) { // Reading more apparently returns old data from previous command
         // TODO: Add ability to return short reads to caller
@@ -398,41 +431,41 @@ nvme_device * smart_interface::get_snt_device(const char * type, scsi_device * s
   if (!scsidev)
     throw std::logic_error("smart_interface: get_snt_device() called with scsidev=0");
 
+  // Check for "snt*/sat"
+  bool maybe_sat = false;
+  char snt_type[32];
+  snprintf(snt_type, sizeof(snt_type), "%s", type);
+  int len = strlen(snt_type);
+  if (len > 4 && !strcmp(snt_type + len - 4, "/sat")) {
+    snt_type[len -= 4] = 0;
+    maybe_sat = true;
+  }
+
   // Take temporary ownership of 'scsidev' to delete it on error
   scsi_device_auto_ptr scsidev_holder(scsidev);
-  nvme_device * sntdev = 0;
+  nvme_device * sntdev = nullptr;
 
-  // TODO: Remove this and adjust drivedb entry accordingly when no longer EXPERIMENTAL
-  if (!strcmp(type, "sntjmicron#please_try")) {
-    set_err(EINVAL, "USB to NVMe bridge [please try '-d sntjmicron' and report result to: "
-            PACKAGE_BUGREPORT "]");
-    return 0;
-  }
-
-  if (!strcmp(type, "sntasmedia")) {
+  if (!strcmp(snt_type, "sntasmedia")) {
     // No namespace supported
-    sntdev = new sntasmedia_device(this, scsidev, type, 0xffffffff);
+    sntdev = new sntasmedia_device(this, scsidev, type, nvme_broadcast_nsid, maybe_sat);
   }
 
-  else if (!strncmp(type, "sntjmicron", 10)) {
-    int n1 = -1, n2 = -1, len = strlen(type);
-    unsigned nsid = 0; // invalid namespace id -> use default
-    sscanf(type, "sntjmicron%n,0x%x%n", &n1, &nsid, &n2);
-    if (!(n1 == len || n2 == len)) {
-      set_err(EINVAL, "Invalid NVMe namespace id in '%s'", type);
-      return 0;
-    }
-    sntdev = new sntjmicron_device(this, scsidev, type, nsid);
+  else if (str_starts_with(snt_type, "sntjmicron")) {
+    int n1 = -1, n2 = -1;
+    unsigned nsid = nvme_broadcast_nsid;
+    sscanf(snt_type, "sntjmicron%n,0x%x%n", &n1, &nsid, &n2);
+    if (!(n1 == len || n2 == len))
+      return set_err_np(EINVAL, "Invalid NVMe namespace id in '%s'", snt_type);
+    sntdev = new sntjmicron_device(this, scsidev, type, nsid, maybe_sat);
   }
 
-  else if (!strcmp(type, "sntrealtek")) {
+  else if (!strcmp(snt_type, "sntrealtek")) {
     // No namespace supported
-    sntdev = new sntrealtek_device(this, scsidev, type, 0xffffffff);
+    sntdev = new sntrealtek_device(this, scsidev, type, nvme_broadcast_nsid, maybe_sat);
   }
 
   else {
-    set_err(EINVAL, "Unknown SNT device type '%s'", type);
-    return 0;
+    return set_err_np(EINVAL, "Unknown SNT device type '%s'", type);
   }
 
   // 'scsidev' is now owned by 'sntdev'
